#include "inkview.h"
#include <vector>
#include <string>

class PBControll;
typedef void (PBControll_cb)(PBControll*,void*val);
typedef PBControll* PBControl_ptr;
class PBControll{
  public:
  int x,y,w,h;
  bool focused;
  bool visible;
  std::string label;
  PBControl_ptr _parent;
  PBControll_cb* _callback;
  static ifont* cfont;
  protected:
  public:
  void do_callback(void*vl){if(_callback)_callback(this,vl);}
  bool inside(int _x,int _y)const{
    return (_x>x && _x<(x+w) && _y>y && _y<(y+h));
  }
  void parent(PBControl_ptr _p){_parent=_p;}
  PBControl_ptr parent()const{return _parent;};
  PBControl_ptr getPtr(){
    return this;
  }
  void callback(PBControll_cb* cb){_callback=cb;}
  PBControll(int X,int Y,int W,int H,const char* L=""):x(X),y(Y),w(W),h(H),focused(false),_callback(NULL){
  label=L;};
  virtual void draw(bool no_redraw=true)=0;
  virtual int handle(int type,int par1,int par2)=0;
  virtual ~PBControll(){};
};
class PBGroup : public PBControll{
  std::vector<PBControl_ptr> children;
  PBControl_ptr focused_children;
  public:
  void add(PBControl_ptr);
  virtual void draw(bool no_redraw=true);
  virtual int handle(int type,int par1,int par2);
  ~PBGroup();
};

class PBSlider : public PBControll{
  int _value,min_val,max_val;
  public:
  int value()const{return _value;};
  void value(int v){_value=v;}
  PBSlider(int X,int Y,int W,int H,int min=0,int max=100):
    PBControll(X,Y,W,H),min_val(min),max_val(max){}
  void draw(bool no_redraw=true){
    FillArea(x,y,w,h,WHITE);
    if(focused)DrawSelection(x,y,w,h,BLACK);
    DrawRect(x+2,y+2,w-4,h-4,BLACK);
    double part=1./(max_val-min_val)*value();
    FillArea(x+4,y+4+(h-8.)*(1.-part),w-8,(h-8.)*part,BLACK);
    if(!no_redraw){
      PartialUpdateBW(x,y,w,h);
    }
  }
  int handle(int type,int par1,int par2){
    if(focused){
      if(EVT_KEYPRESS==type){
        switch(par1){
          case KEY_UP:
          ++_value;if(_value>max_val)_value=max_val;
          do_callback((void*)_value);
          draw(false);
          return 1;
          case KEY_DOWN:
          --_value;if(_value<min_val)_value=min_val;
          do_callback((void*)_value);
          draw(false);
          return 1;
        }
        return 0;
      }
    }
    if(EVT_POINTERDOWN==type){
      if(inside(par1,par2)){
        _value=double(h-(par2-y))/h*(max_val-min_val);
        do_callback((void*)_value);
        focused=true;
      }else{
        focused=false;
      }
    }
    draw(false);
    return 0;
  }
};
typedef std::vector<std::string> FillItemsFunc();
class PBMenuButton : public PBControll{
  imenu* _menu;
  const char* default_label;
  FillItemsFunc* _itemfunc;
  std::vector<std::string> menus;
  static PBMenuButton* activemenu;
  static void menu_handler(int item);
  void do_menu();
  public:
  void menu(FillItemsFunc* itemfunc){_itemfunc=itemfunc;}
  PBMenuButton(int X,int Y,int W,int H,const char* L=""):PBControll(X,Y,W,H,L),_menu(NULL){
  }
  void draw(bool no_redraw=true){
    FillArea(x,y,w,h,WHITE);
    if(focused)DrawSelection(x,y,w,h,BLACK);
    DrawRect(x+2,y+2,w-4,h-4,BLACK);
    if(cfont){
      SetFont(cfont,BLACK);
      DrawString(x,y+5,label.c_str());
    }
    if(!no_redraw){
      PartialUpdateBW(x,y,w,h);
    }
  }
  int handle(int type,int par1,int par2){
    if(focused){
      if(EVT_KEYPRESS==type){
        switch(par1){
          case KEY_OK:
          do_menu();
          draw(false);
          return 1;
        }
        return 0;
      }
    }
    if(EVT_POINTERDOWN==type){
      if(inside(par1,par2)){
        do_menu();
        focused=true;
      }else{
        focused=false;
      }
    }
    draw(false);
    return 0;
  }
};

